function bimolStochastic
S = [ -1   1       
      -1   1      
       1  -1   
       ];
V = 1e-15;                                       
Na = 6.023e23;   
n0 = [1000;1000;0];
L = -S.*(S < 0);   
tf = 10;        
c=[1.78e-4 0.351];
runs = 1; 
[ts,ns] = stoch(n0,c,S,L,tf);                  
hold on;
plot(ts,ns(1,:),'r','LineWidth',3);
plot(ts,ns(2,:),'b');
plot(ts,ns(3,:),'g');
legend('A','B','C');
xlabel('Time (sec)','FontSize',20);
ylabel('Particle count','FontSize',20);
function varargout = stoch(varargin)
%
%   
%   N0: Column Vector of initial populations of all the species involved 
%   C: Row vector of stochastic rate constants of all elementary reactions
%   S: Stoichiometry matrix with rows correspoding to species and columns
%      corresponding to reaction channels
%   L: Stoichiometry matrix for reactants only such that L = -D.*(D < 0); 
%   TF: Final time of simulation    
%   Ts: Row vector of time points of reaction events
%   Ns: Matrix of output concentrations with a column for each time point.

args = varargin;                        
if nargin<6, args{6} = 1; end           % single run by default
[n0,c,d,l,tf,runs] = args{:};           % parse inputs
oneszk = ones(size(c));
i1 = l==1;                              % reactions of type: A->B, A+B->AB
i2 = l==2;                              % reactions of type: A+A->AA
stop = tf - eps(tf);                    % simulation stop time
nOut = nargout;

% stochastic part:    

if runs < 2                                        % single run
    [yout{1:2}] = gillespie;                       % run gillispie
else                                               % multiple runs
    [tg,ng] = deal(cell(1,runs));
    for i = 1:runs, [tg{i},ng{i}] = gillespie; end % run gillispie
    yout = {tg,ng};      
    if nOut>2
        tt = unique([tg{:}]);                      % record times
        numeltt = numel(tt);                       % record populations
        nn = zeros(numel(n0), numeltt, runs);
        for i = 1:numeltt
            for j = 1:runs
                id = find(tg{j} <= tt(i),1,'last');
                nn(:,i,j) = ng{j}(:,id);
            end
        end
        yout(3:4) = {tt,mean(nn,3)};               % append population mean 
    end
end

% Output:

varargout = yout;

%   gillespie algorithm (direct method):

    function [tt,nn] = gillespie
        t = 0;                              % initial time
        n = n0;                             % initial population
        tt = [];
        nn = [];
        rand('state', sum(100*clock));      % reset random number generator
        while t <= stop
            tt = [tt t];                    % record time
            nn = [nn n];                    % record population            
            m = n(:,oneszk);                % replicate n : size(m,2) = size(k)
            b = double(~l);
            b(i1) = m(i1);                  % reactions of type: A->B, A+B->AB
            b(i2) = m(i2).*(m(i2)-1)/2;     % reactions of type: A+A->AA
            a = c.*prod(b);                 % propensity
            astr = sum(a); 
            if ~astr, break, end            % substrate utilised
            tau = -1/astr*log(rand);        % time to next reaction
            u = find(cumsum(a)>astr*rand,1);% index of next reaction
            n = n + d(:,u);                 % update population
            t = t + tau;                    % update time
        end
        tt = [tt tf];
        nn = [nn nn(:,end)];
    end

end
end